/// <reference types="node" />
import * as anchor from "@project-serum/anchor";
import { IdlTypes, Program, Provider } from "@project-serum/anchor";
import { AccountInfo as TokenAccountInfo } from "@solana/spl-token";
import { AccountInfo, PublicKey } from "@solana/web3.js";
import { ICreateTokenBondingArgs, SplTokenBonding } from "@strata-foundation/spl-token-bonding";
import { AnchorSdk, BigInstructionResult, InstructionResult, ITokenWithMeta, SplTokenMetadata, TypedAccountParser } from "@strata-foundation/spl-utils";
import { CollectiveV0, SplTokenCollectiveIDL, TokenRefV0 } from "./generated/spl-token-collective";
export * from "./generated/spl-token-collective";
export interface ITokenWithMetaAndAccount extends ITokenWithMeta {
    publicKey?: PublicKey;
    tokenRef?: ITokenRef;
    account?: TokenAccountInfo;
}
interface TokenAccount {
    pubkey: PublicKey;
    account: AccountInfo<Buffer>;
    info: TokenAccountInfo;
}
export interface ICreateCollectiveArgs {
    /** Payer for this transaction */
    payer?: PublicKey;
    /**
     * Token metadata that, if provided, will create metaplex spl-token-metadata for this collective.
     *
     * Reccommended to always fill this out so that your token displays with a name, symbol, and image.
     */
    metadata?: {
        name: string;
        symbol: string;
        uri: string;
    };
    /**
     * If `mint` is not provided, create a bonding curve automatically for this collective.
     */
    bonding?: ICreateTokenBondingArgs;
    /** The mint to base this collective around. It is recommended for compatability that all collectives be on a bonding curve, so it's easy to make user interfaces that can buy in and out of your social tokens */
    mint?: PublicKey;
    /** **Default:** Fetch from mint. This may not be possible if the mint is being created in the same transaction as the collective. */
    mintAuthority?: PublicKey;
    /** The authority of this collective */
    authority?: PublicKey;
    /** The configs around what is and isn't allowed in the collective */
    config: ICollectiveConfig;
}
/** See [InitializeTokenBondingArgs](/docs/api/spl-token-bonding/interfaces/ICreateTokenBondingArgs) */
export interface ITokenBondingParams {
    /** The curve to create this social token on. **Default:** Curve from the collective's config */
    curve?: PublicKey;
    buyBaseRoyaltyPercentage: number;
    buyTargetRoyaltyPercentage: number;
    sellBaseRoyaltyPercentage: number;
    sellTargetRoyaltyPercentage: number;
    /** **Default:** uses decimals from collective config, or 9 */
    targetMintDecimals?: number;
    buyBaseRoyalties?: PublicKey;
    buyBaseRoyaltiesOwner?: PublicKey;
    buyTargetRoyalties?: PublicKey;
    buyTargetRoyaltiesOwner?: PublicKey;
    sellBaseRoyalties?: PublicKey;
    sellBaseRoyaltiesOwner?: PublicKey;
    sellTargetRoyalties?: PublicKey;
    sellTargetRoyaltiesOwner?: PublicKey;
}
/**
 * Set this token as your primary token, so people can look you up without knowing the collective
 */
export interface ISetAsPrimaryArgs {
    payer?: PublicKey;
    tokenRef: PublicKey;
    /**
     * The owner of the `tokenRef`. **Default:** Owner from fetching tokenRef. You may need to provide this if setting
     * primary in the same txn as creating the token ref.
     */
    owner?: PublicKey;
}
/**
 * Update this collective
 */
export interface IUpdateCollectiveArgs {
    payer?: PublicKey;
    collective: PublicKey;
    /**
     * The authority `collective`. **Default:** Authority from fetching the collective.
     *
     * Explicitly pass null to set the authority to none
     */
    authority?: PublicKey | null;
    config: ICollectiveConfig;
}
export interface ICreateSocialTokenArgs {
    /**
     * Is this the primary social token for this wallet? **Default:** true
     *
     * A primary social token is the social token people should see when they look up your wallet. While it's possible to belong to many
     * collectives, generally most people will have one social token.
     *
     * This can be changed at any time.
     */
    isPrimary?: boolean;
    /** If this social token already exists, don't throw an error. **Default:** false */
    ignoreIfExists?: boolean;
    /** The payer for this account and txn */
    payer?: PublicKey;
    /** The collective to create this social token under. **Default:**: the Open Collective*/
    collective?: PublicKey;
    /** The spl-name-service name to associate with this account. Will create an unclaimed social token. */
    name?: PublicKey;
    /** The spl-name-service name class associated with name above, if provided */
    nameClass?: PublicKey;
    /** The spl-name-service name paent associated with name above, if provided */
    nameParent?: PublicKey;
    /**
     * Token metadata that, if provided, will create metaplex spl-token-metadata for this collective.
     *
     * Reccommended to fill this out so that your token displays with a name, symbol, and image.
     */
    metadata: {
        name: string;
        symbol: string;
        /** Getting a URI for token metadata can be an expensive process that involves a separate transaction
         * If the collective has a default URI configured, you can just not pass this
         * **Default:** {@link ICollectiveConfig.unclaimedTokenMetadataSettings.uri} */
        uri?: string;
    };
    /** The wallet to create this social token under, defaults to `provider.wallet` */
    owner?: PublicKey;
    /**
     * **Default:** New generated keypair
     *
     * Pass in the keypair to use for the mint. Useful if you want a vanity keypair
     */
    targetMintKeypair?: anchor.web3.Keypair;
    /** Params for the bonding curve  */
    tokenBondingParams: ITokenBondingParams;
}
export interface IClaimSocialTokenArgs {
    /**
     * Is this the primary social token for this wallet? **Default:** true
     *
     * A primary social token is the social token people should see when they look up your wallet. While it's possible to belong to many
     * collectives, generally most people will have one social token.
     */
    isPrimary?: boolean;
    /** The payer for this txn */
    payer?: PublicKey;
    /** The owning wallet of this social token. **Default:**: `provider.wallet` */
    owner?: PublicKey;
    /** The token ref of the token we are claiming */
    tokenRef: PublicKey;
    /** Change the smart-contract level name for this token without changing the url. To do a full update to token metadata, directly use SplTokenMetadata after a claim */
    tokenName?: string;
    /** Change the smart-contract level symbol for this token without changing the url. To do a full update to token metadata, directly use SplTokenMetadata after a claim */
    symbol?: string;
    /** The buy base royalties destination. **Default:** ATA of owner */
    buyBaseRoyalties?: PublicKey;
    /** The buy target royalties destination. **Default:** ATA of owner */
    buyTargetRoyalties?: PublicKey;
    /** The sell base royalties destination. **Default:** ATA of owner */
    sellBaseRoyalties?: PublicKey;
    /** The sell target royalties destination. **Default:** ATA of owner */
    sellTargetRoyalties?: PublicKey;
    /**
     * Ignore missing name account. Useful if you're creating the name in the same txn.
     *
     * Otherwise, the sdk checks to make sure the name account exists before claiming to provide a more useful error
     *
     * **Default:** false
     */
    ignoreMissingName?: boolean;
}
interface ITokenRefKeyArgs {
    isPrimary?: boolean;
    owner?: PublicKey;
    name?: PublicKey;
    collective?: PublicKey;
}
export interface IRoyaltySetting {
    /**
     * In the case of an unclaimed token, is this royalty account required to be owned by the name account.
     *
     * If `true`, when the token is claimed, the owner of the name that's claiming it will receive all of the funds in the royalty account
     */
    ownedByName?: boolean;
    /**
     * A static address such that all curves must have this as the royalty address.
     */
    address?: number;
}
export interface ITokenBondingSettings {
    curve?: PublicKey;
    minSellBaseRoyaltyPercentage?: number;
    minSellTargetRoyaltyPercentage?: number;
    maxSellBaseRoyaltyPercentage?: number;
    maxSellTargetRoyaltyPercentage?: number;
    minBuyBaseRoyaltyPercentage?: number;
    minBuyTargetRoyaltyPercentage?: number;
    maxBuyBaseRoyaltyPercentage?: number;
    maxBuyTargetRoyaltyPercentage?: number;
    targetMintDecimals?: number;
    buyBaseRoyalties?: IRoyaltySetting;
    sellBaseRoyalties?: IRoyaltySetting;
    buyTargetRoyalties?: IRoyaltySetting;
    sellTargetRoyalties?: IRoyaltySetting;
    minPurchaseCap?: number;
    maxPurchaseCap?: number;
    minMintCap?: number;
    maxMintCap?: number;
}
export interface ITokenMetadataSettings {
    /** The default symbol for an unclaimed token */
    symbol?: string;
    /** The default uri for an unclaimed token */
    uri?: string;
    /** Enforce that the name of the unclaimed token matches the spl-name-service name */
    nameIsNameServiceName?: boolean;
}
export interface ICollectiveConfig {
    /**
     * A collective can either be open or closed. A closed collective must sign on the creation of _any_ social token
     * within the collective. An open collective allows any social tokens to bind themself to the collective token, so long
     * as they follow the CollectiveConfig settings
     */
    isOpen: boolean;
    /** Settings for bonding curves on unclaimed tokens */
    unclaimedTokenBondingSettings?: ITokenBondingSettings;
    /** Settings for bonding curves on claimed tokens */
    claimedTokenBondingSettings?: ITokenBondingSettings;
    /** Settings for token metadata of unclaimed tokens */
    unclaimedTokenMetadataSettings?: ITokenMetadataSettings;
}
declare type CollectiveConfigV0 = IdlTypes<SplTokenCollectiveIDL>["CollectiveConfigV0"];
/**
 * Unified tokenref interface wrapping the raw TokenRefV0
 */
export interface ITokenRef extends TokenRefV0 {
    publicKey: PublicKey;
}
/**
 * Unified collective interface wrapping the raw CollectiveV0
 */
export interface ICollective extends CollectiveV0 {
    publicKey: PublicKey;
    config: CollectiveConfigV0;
}
export declare class SplTokenCollective extends AnchorSdk<SplTokenCollectiveIDL> {
    splTokenBondingProgram: SplTokenBonding;
    splTokenMetadata: SplTokenMetadata;
    static ID: anchor.web3.PublicKey;
    static OPEN_COLLECTIVE_ID: anchor.web3.PublicKey;
    static OPEN_COLLECTIVE_BONDING_ID: anchor.web3.PublicKey;
    static OPEN_COLLECTIVE_MINT_ID: anchor.web3.PublicKey;
    static init(provider: Provider, splCollectiveProgramId?: PublicKey, splTokenBondingProgramId?: PublicKey): Promise<SplTokenCollective>;
    constructor(opts: {
        provider: Provider;
        program: Program<SplTokenCollectiveIDL>;
        splTokenBondingProgram: SplTokenBonding;
        splTokenMetadata: SplTokenMetadata;
    });
    /**
     * Account decoder to a unified TokenRef interface
     *
     * @param pubkey
     * @param account
     * @returns
     */
    tokenRefDecoder: TypedAccountParser<ITokenRef>;
    /**
     * Account decoder to a unified Collective interface
     *
     * @param pubkey
     * @param account
     * @returns
     */
    collectiveDecoder: TypedAccountParser<ICollective>;
    getCollective(collectiveKey: PublicKey): Promise<ICollective | null>;
    getTokenRef(tokenRefKey: PublicKey): Promise<ITokenRef | null>;
    /**
     * Instructions to create a Collective
     *
     * @param param0
     * @returns
     */
    createCollectiveInstructions({ payer, mint, authority, mintAuthority, config, bonding, metadata, }: ICreateCollectiveArgs): Promise<BigInstructionResult<{
        collective: PublicKey;
        tokenBonding?: PublicKey;
    }>>;
    /**
     * Run {@link createCollectiveInstructions}
     * @param args
     * @returns
     */
    createCollective(args: ICreateCollectiveArgs): Promise<{
        collective: PublicKey;
        tokenBonding?: PublicKey;
    }>;
    /**
     * Instructions to claim a social token
     *
     * @param param0
     * @returns
     */
    claimSocialTokenInstructions({ payer, owner, tokenRef, tokenName, symbol, buyBaseRoyalties, buyTargetRoyalties, sellBaseRoyalties, sellTargetRoyalties, ignoreMissingName, isPrimary, }: IClaimSocialTokenArgs): Promise<InstructionResult<null>>;
    /**
     * Run {@link claimSocialTokenInstructions}
     * @param args
     */
    claimSocialToken(args: IClaimSocialTokenArgs): Promise<void>;
    /**
     * Get the seeds for the PDA of a token ref given the various parameters.
     *
     * @param param0
     * @returns
     */
    static tokenRefSeeds({ owner, name, collective, isPrimary, }: ITokenRefKeyArgs): Buffer[];
    static tokenRefKey(args: ITokenRefKeyArgs, programId?: PublicKey): Promise<[PublicKey, number]>;
    static reverseTokenRefKey(mint: PublicKey, programId?: PublicKey): Promise<[PublicKey, number]>;
    /**
     * Get instructions to set this tokenRef as our primary token ref (lookups to "token-ref", owner pda find this tokenRef)
     *
     * @param param0
     * @returns
     */
    setAsPrimaryInstructions({ payer, tokenRef, owner, }: ISetAsPrimaryArgs): Promise<InstructionResult<{
        primaryTokenRef: PublicKey;
    }>>;
    /**
     * Run {@link setAsPrimaryInstructions}
     * @param args
     */
    setAsPrimary(args: ISetAsPrimaryArgs): Promise<{
        primaryTokenRef: PublicKey;
    }>;
    /**
     * Get instructions to update this collective
     *
     * @param param0
     * @returns
     */
    updateCollectiveInstructions({ collective, authority, config, }: IUpdateCollectiveArgs): Promise<InstructionResult<null>>;
    /**
     * Run {@link updateCollectiveInstructions}
     * @param args
     */
    updateCollective(args: IUpdateCollectiveArgs): Promise<null>;
    /**
     * Instructions to create everything around a social token... metadata, bonding curves, etc.
     *
     * @param param0
     * @returns
     */
    createSocialTokenInstructions({ ignoreIfExists, payer, collective, name, owner, targetMintKeypair, metadata, nameClass, nameParent, tokenBondingParams, isPrimary, }: ICreateSocialTokenArgs): Promise<BigInstructionResult<{
        tokenRef: PublicKey;
        reverseTokenRef: PublicKey;
        tokenBonding: PublicKey;
        mint: PublicKey;
    }>>;
    /**
     * Run {@link createSocialTokenInstructions}
     * @param args
     * @returns
     */
    createSocialToken(args: ICreateSocialTokenArgs): Promise<{
        tokenRef: PublicKey;
        reverseTokenRef: PublicKey;
        tokenBonding: PublicKey;
        mint: PublicKey;
    }>;
    getUserTokensWithMeta(tokenAccounts?: TokenAccount[]): Promise<ITokenWithMetaAndAccount[]>;
}
//# sourceMappingURL=index.d.ts.map